#include <QTcpSocket>
#include <QHttp>
// (c) Thomas Kamps, 2009
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the7 implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include <QHttpResponseHeader>
#include <QHttpRequestHeader>
#include <QDomDocument>
#include <QDomElement>
#include <QMessageBox>
#include <QTableWidgetItem>
#include "monopdgame.h"
#include "monopdfield.h"
#include "monopdclient.h"
#include "ui_monopdclient.h"

MonopdClient::MonopdClient(QTcpSocket* sok, QWidget *parent) : QWidget(parent), m_ui(new Ui::MonopdClient) {
	m_ui->setupUi(this);
	soket = sok;

	connect(sok, SIGNAL(readyRead()), this, SLOT(readData()));
	connect(m_ui->pushButton, SIGNAL(clicked()), this, SLOT(sendData()));
	connect(sok, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(showError()));

	curGameId = -1;

	httpSok = new QHttp();
	httpSok->setHost("monopd-gator.kde.org");
	httpSok->request(QHttpRequestHeader("GET / HTTP/1.1\nHost: monopd-gator.kde.org\nConnection: Keep-Alive\nUser-Agent: Capitalism 0.1\nPragma: no-cache\nCache-control: no-cache\nAccept: text/html, text/*;q=0.9, */*;q=0.8\nAccept-Encoding: x-gzip, x-deflate, gzip, deflate\nAccept-Charset: utf-8, utf-8;q=0.5, *;q=0.5\nAccept-Language: de, en-US, en\n"));

	connect(httpSok, SIGNAL(readyRead(QHttpResponseHeader)), this, SLOT(readServerList()));

	connect(m_ui->tServer, SIGNAL(cellClicked(int,int)), this, SLOT(setServer(int,int)));
	connect(m_ui->bConnect, SIGNAL(clicked()), this, SLOT(connectToServer()));
}

MonopdClient::~MonopdClient() {
	delete m_ui;
}

//Returns a Player. If no one is found, onbe will be created
Player* MonopdClient::getPlayer(int pid) {
	Player* p = 0;
	for (int i = 0; i < players.size(); i++)
		if (players.value(i)->getId() == pid) return players.value(i);
	p = new Player(pid);
	players.append(p);
	return p;
}

bool MonopdClient::hasPlayer(int pid) {
	for (int i = 0; i < players.size(); i++)
		if (players.value(i)->getId() == pid) return true;
	return false;
}

//Returns a Game template. If no one is found, onbe will be created
MonopdTemplate* MonopdClient::getTemplate(QString t) {
	MonopdTemplate* temp = 0;
	for (int i = 0; i < templates.size(); i++)
		if (templates.value(i)->getType() == t)
			return templates.value(i);
	temp = new MonopdTemplate(t, "", "");
	templates.append(temp);
	return temp;
}

MonopdTemplate* MonopdClient::getTemplateFromList(int n) {
	if (n < 0) return 0;
	if (n >= templates.size()) return 0;
	return templates.value(n);
}

//Returns a Game. If no one is found, onbe will be created if type is given or created is true, otherwise 0 is returned
MonopdGame* MonopdClient::getGame(int gid, QString type, bool create) {
	MonopdGame* g = 0;
	for (int i = 0; i < games.size(); i++)
		if (games.value(i)->getGameId() == gid) return games.value(i);
	if ((type == "") && !create) return 0;
	g = new MonopdGame(type, "", "", gid);
	games.append(g);
	return g;
}

MonopdGame* MonopdClient::getGameFromList(int n) {
	if (n < 0) return 0;
	if (n >= games.size()) return 0;
	return games.value(n);
}

int MonopdClient::getNumTemplates() {
	return templates.size();
}

int MonopdClient::getNumGames() {
	return games.size();
}

int MonopdClient::getCurGameId() {
	return curGameId;
}

int MonopdClient::getClientPlayerId() {
	return clientPlayerId;
}

QString MonopdClient::getClientCookie() {
	return clientCookie;
}

//Deletes the player. It will be removed from all games
void MonopdClient::deletePlayer(Player* p) {
	for (int i = 0; i < games.size(); i++)
		games.value(i)->removePlayer(p);
	players.removeAll(p);
	p->~Player();
}

void MonopdClient::deleteGame(MonopdGame* g) {
	if (g == 0) return;
	games.removeAll(g);
	g->~MonopdGame();
}

void MonopdClient::setCurGameId(int id) {
	curGameId = id;
}

void MonopdClient::setClientPlayerId(int id) {
	clientPlayerId = id;
}

void MonopdClient::setClientCookie(QString c) {
	clientCookie = c;
}

void MonopdClient::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	switch (e->type()) {
		case QEvent::LanguageChange:
			m_ui->retranslateUi(this);
			break;
		default:
			break;
	}
}

void MonopdClient::parseDoc(QDomDocument d) {
	QDomElement root = d.documentElement();
	if (root.tagName() == "monopd") {
		bool update = false;
		QDomElement e = root.firstChildElement();
		while (!e.isNull()) {
			QString name = e.nodeName();
			if (name == "estateupdate") {
				parseField(e);
				update = true;
			}
			if (name == "cardupdate") {
				parseCard(e);
			}
			if (name == "tradeupdate") {
				parseTrade(e);
			}
			if (name == "playerupdate") {
				parsePlayer(e);
			}
			if (name == "msg") {
				parseMsg(e);
			}
			if (name == "deleteplayer") {
				bool ok;
				int pid = e.attribute("playerid", "").toInt(&ok);
				if (ok) deletePlayer(getPlayer(pid));
			}
			if (name == "deltegame") {
				bool ok;
				int gid = e.attribute("gameid", "").toInt(&ok);
				if (ok) deleteGame(getGame(gid));
			}
			if (name == "client") {
				bool ok;
				int pid = e.attribute("playerid", "").toInt(&ok);
				if (ok) {
					clientPlayerId = pid;
					emit(playerAdded(getPlayer(pid)));
				}
				clientCookie = e.attribute("cookie", "");
			}
			if (name == "gameupdate") {
				parseGame(e);
			}
			if (name == "updategamelist") {
				QString type = e.attribute("type", "");
				//Only implementet for compatiblity with monopd 0.9.3
				QDomElement gu = e.firstChildElement("game");
				while (!gu.isNull()) {
					parseGamelistUpdate(gu);
					gu = e.nextSiblingElement("game");
				}
			}
			if (name == "configupdate") {
				parseConfig(e);
			}
			if (name == "display") {
				parseDisplay(e);
			}
			if (name == "auctionupdate") {
				parseAuction(e);
			}
			e = e.nextSiblingElement();
		}
		if (update) emit(fieldUpdate());
		update = false;

		//Set visibility of all known players
		for (int i = 0; i < players.size(); i++) {
			Player* p = players.value(i);
			p->setPlayerWidgetVisible(curGameId == p->getGame());
		}
		if (curGameId == -1) emit(newGameList());
	} else if (root.tagName() == "monopigator") {
		//We get a list of servers
		QDomElement e = root.firstChildElement("server");
		while (!e.isNull()) {
			QString host = e.attribute("host", "");
			QTableWidgetItem* tih = new QTableWidgetItem(host);
			if (host != "") {
				QTableWidgetItem* tiport = new QTableWidgetItem(e.attribute("port", ""));
				QTableWidgetItem* tip = new QTableWidgetItem(e.attribute("users", ""));
				QTableWidgetItem* tiip = new QTableWidgetItem(e.attribute("ip", ""));
				QTableWidgetItem* tiv = new QTableWidgetItem(e.attribute("version", ""));
				int r = m_ui->tServer->rowCount();
				tih->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
				tiport->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
				tip->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
				tiip->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
				tiv->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
				m_ui->tServer->insertRow(r);
				m_ui->tServer->setItem(r, 0, tih);
				m_ui->tServer->setItem(r, 1, tiport);
				m_ui->tServer->setItem(r, 2, tip);
				m_ui->tServer->setItem(r, 3, tiip);
				m_ui->tServer->setItem(r, 4, tiv);
			}
			e = e.nextSiblingElement("server");
		}
	}
}

void MonopdClient::parseMsg(QDomElement m) {
	QString type = m.attribute("type", "NaN");
	QString msg = m.attribute("value", "");
	QString val;
	bool ok;
	if (type == "NaN") return;
	if (type == "chat") {
		//a simple chat message
		val = m.attribute("playerid", "");
		if (val == "") return;
		int pid = val.toInt(&ok);
		if (!ok) return;
		emit(chatMessage(pid, msg));
	} else {
		emit(serverMessage(type, msg));
	}
}

void MonopdClient::parseDisplay(QDomElement d) {
	QString val;
	bool ok;
	int fid;

	val = d.attribute("estateid", "NaN");
	fid = val.toInt(&ok);
	if (!ok) return;

	//Check FieldID
	if (fid == -1) emit(clearedText());

	//Check if test should be cleared
	val = d.attribute("cleartext", "");
	if (val == "1") emit(clearedText());

	val = d.attribute("text", "");
	if (val != "") emit(gotText(val));

	//Parse Buttons
	QDomElement b = d.firstChildElement("button");
	while (!b.isNull()) {
		bool enabled = false;
		QString text;
		QString cmd;
		val = b.attribute("enabled", "");
		if (val == "1") enabled = true;

		text = b.attribute("caption", "");

		cmd = b.attribute("command", "");

		if (enabled && (cmd != "")) {
			//if (!cmd.startsWith(".e")) {
				emit(gotButton(text, cmd));
			//}
		}

		b = b.nextSiblingElement("button");
	}

}

void MonopdClient::parseGamelistUpdate(QDomElement g) {
	QString val;
	int gameId;
	bool ok;

	gameId = g.attribute("id", "NaN").toInt(&ok);
	if (!ok) return;
	if (gameId == -1) {
		MonopdTemplate* temp = getTemplate(g.attribute("gametype", ""));
		if (temp == 0) return;

		temp->setDesc(g.attribute("description", ""));

		temp->setName(g.attribute("name", ""));
	} else {
		MonopdGame* game = getGame(gameId, g.attribute("gametype", ""));
		if (game == 0) return;

		int players = g.attribute("players", "").toInt(&ok);
		if (ok) game->setNumPlayer(players);

		val = g.attribute("description", "");
		if (val != "") game->setDesc(val);

		val = g.attribute("name", "");
		if (val != "") game->setDesc(val);

		bool cbj = false;
		if (g.attribute("canbejoined", "") == "1") cbj = true;
		game->setCanBeJoined(cbj);
	}
}

void MonopdClient::parsePlayer(QDomElement f) {
	QString val;
	int id;
	bool ok;

	val = f.attribute("playerid", "NaN");
	if (val == "NaN") return;
	id = val.toInt(&ok);
	if (!ok) return;
	if (id < 0) return;
	bool toAdd = !hasPlayer(id);
	Player* p = getPlayer(id);
	if (toAdd) emit(playerAdded(p));

	val = f.attribute("game", "");
	if (val != "") {
		int g = val.toInt(&ok);
		if (ok) {
			if (p->getId() == clientPlayerId) {
				int oldGid = curGameId;
				curGameId = g;
				if (oldGid != g) emit(curGameChanged());
			}
			if (p->getGame() != g) {
				if (p->getGame() != -1) {
					MonopdGame* game = getGame(p->getGame());
					if (game != 0) game->removePlayer(p);
				}
				if (g != -1) {
					MonopdGame* game = getGame(g);
					if (game != 0) game->addPlayer(p);
				}
			}
			p->setGame(g);
		}
	}

	val = f.attribute("name", "");
	if (val != "") {
		p->setName(val);
	}

	val = f.attribute("image", "");
	if (val != "") {
		p->setAvatar(val);
		emit(fieldUpdate());
	}

	val = f.attribute("money", "");
	if (val != "") {
		int m = val.toInt(&ok);
		if (ok) {
			p->setMoney(m);
		}
	}

	val = f.attribute("location", "");
	if (val != "") {
		int pos = val.toInt(&ok);
		if (ok) {
			p->setPos(pos);
			send(".t"+QString::number(pos));
			emit(fieldUpdate());
		}
	}

	val = f.attribute("jailed", "");
	if (val != "") {
		bool j = false;
		if (val == "1") j = true;
		p->setInJail(j);
	}

//	val = f.attribute("directmove", "");
//	if (val != "") {
//		bool dm = false;
//		if (val == "1") dm = true;
//		fld->setDirectMove(dm);
//	}

	val = f.attribute("bankrupt", "");
	if (val != "") {
		bool b = false;
		if (val == "1") b = true;
		p->setBankrupt(b);
	}

	val = f.attribute("hasturn", "");
	if (val != "") {
		bool ht = false;
		if (val == "1") ht = true;
		p->setHasTurn(ht);
	}

	val = f.attribute("can_roll", "");
	if (val != "") {
		bool cr = false;
		if (val == "1") cr = true;
		p->setCanRoll(cr);
	}

	val = f.attribute("can_buyestate", "");
	if (val != "") {
		bool cbe = false;
		if (val == "1") cbe = true;
		p->setCanBuyEstate(cbe);
	}

	val = f.attribute("host", "");
	if (val != "") {
		p->setHost(val);
	}

	val = f.attribute("master", "");
	if (val != "") {
		bool m = false;
		if (val == "1") m = true;
		p->setMaster(m);
	}

	val = f.attribute("spectator", "");
	if (val != "") {
		bool s = false;
		if (val == "1") s = true;
		p->setSpectator(s);
	}

	val = f.attribute("hasdept", "");
	if (val != "") {
		bool hd = false;
		if (val == "1") hd = true;
		p->setHasDept(hd);
	}

	val = f.attribute("canauction", "");
	if (val != "") {
		bool ca = false;
		if (val == "1") ca = true;
		p->setCanAuction(ca);
	}

	val = f.attribute("canusecard", "");
	if (val != "") {
		bool cuc = false;
		if (val == "1") cuc = true;
		p->setCanUseCard(cuc);
	}

	if (p->getId() == clientPlayerId) emit(playerUpdated());
}

void MonopdClient::parseField(QDomElement f) {
	QString val;
	int id;
	bool ok;

	val = f.attribute("estateid", "NaN");
	if (val == "NaN") return;
	id = val.toInt(&ok);
	if (!ok) return;
	if (id < 0) return;
	MonopdField* fld = getGame(curGameId)->getField(id);

	val = f.attribute("color", "");
	if (val != "") fld->setColor(QColor(val));

	val = f.attribute("owner", "");
	if (val != "") {
		int pid = val.toInt(&ok);
		if (ok) {
			fld->setOwner(pid);
		}
	}

	val = f.attribute("houseprice", "");
	if (val != "") {
		int hp = val.toInt(&ok);
		if (ok) {
			fld->setHousePrice(hp);
		}
	}

	val = f.attribute("group", "");
	if (val != "") {
		int g = val.toInt(&ok);
		if (ok) {
			fld->setGroup(g);
		}
	}

	val = f.attribute("can_be_owned", "");
	if (val != "") {
		bool b = false;
		if (val == "1") b = true;
		fld->setBuyable(b);
	}

	val = f.attribute("can_toggle_mortgage", "");
	if (val != "") {
		bool ctm = false;
		if (val == "1") ctm = true;
		fld->setCanToggleMortage(ctm);
	}

	val = f.attribute("can_buy_houses", "");
	if (val != "") {
		bool cbh = false;
		if (val == "1") cbh = true;
		fld->setCanBuyHouse(cbh);
	}

	val = f.attribute("can_sell_houses", "");
	if (val != "") {
		bool csh = false;
		if (val == "1") csh = true;
		fld->setCanSellHouse(csh);
	}

	val = f.attribute("price", "");
	if (val != "") {
		int p = val.toInt(&ok);
		if (ok) {
			fld->setPrice(p);
		}
	}

	val = f.attribute("rent0", "");
	if (val != "") {
		int r = val.toInt(&ok);
		if (ok) {
			fld->setRent(0,r);
		}
	}

	val = f.attribute("rent1", "");
	if (val != "") {
		int r = val.toInt(&ok);
		if (ok) {
			fld->setRent(1,r);
		}
	}

	val = f.attribute("rent2", "");
	if (val != "") {
		int r = val.toInt(&ok);
		if (ok) {
			fld->setRent(2,r);
		}
	}

	val = f.attribute("rent3", "");
	if (val != "") {
		int r = val.toInt(&ok);
		if (ok) {
			fld->setRent(3,r);
		}
	}

	val = f.attribute("rent4", "");
	if (val != "") {
		int r = val.toInt(&ok);
		if (ok) {
			fld->setRent(4,r);
		}
	}

	val = f.attribute("rent5", "");
	if (val != "") {
		int r = val.toInt(&ok);
		if (ok) {
			fld->setRent(5,r);
		}
	}

	val = f.attribute("name", "");
	if (val != "") {
		fld->setName(val);
	}

	val = f.attribute("houses", "");
	if (val != "") {
		int h = val.toInt(&ok);
		if (ok) {
			fld->setHouses(h);
		}
	}

	//val = f.attribute("money", "");
	//if (val != "") {
		//int m = val.toInt(&ok);
		//if (ok) {
			//fld->setMoney(m);
		//}
	//}

	val = f.attribute("mortgageprice", "");
	if (val != "") {
		int mv = val.toInt(&ok);
		if (ok) {
			fld->setMortageVal(mv);
		}
	}

	val = f.attribute("unmortgageprice", "");
	if (val != "") {
		int umv = val.toInt(&ok);
		if (ok) {
			fld->setUnmortageVal(umv);
		}
	}

	val = f.attribute("sellhouseprice", "");
	if (val != "") {
		int shp = val.toInt(&ok);
		if (ok) {
			fld->setSellHousePrice(shp);
		}
	}

	val = f.attribute("mortgaged", "");
	if (val != "") {
		bool m = false;
		if (val == "1") m = true;
		fld->setMortaged(m);
	}
}

void MonopdClient::parseCard(QDomElement c) {
	QString val;
	bool ok;

	//Get the card
	int id;
	val = c.attribute("cardid", "NaN");
	if (val == "NaN") return;
	id = val.toInt(&ok);
	if (!ok) return;
	MonopdCard* card = getGame(curGameId)->getCard(id);

	//Get title
	val = c.attribute("title", "");
	if (val != "") card->setText(val);

	//Get owner
	val = c.attribute("owner", "");
	if (val != "") {
		int owner = val.toInt(&ok);
		if (ok) card->setOwner(owner);
	}
}

void MonopdClient::parseTrade(QDomElement t) {
	QString val;
	bool ok;

	QString type = "";
	int tradeId = 0;
	int actorId = -1;
	int rev = 0;

	//Get data about the trade
	type = t.attribute("type", "edit");

	tradeId = t.attribute("tradeid", "NaN").toInt(&ok);
	if (!ok) return;

	actorId = t.attribute("actor", "NaN").toInt(&ok);
	if (!ok) actorId = -1;

	rev = t.attribute("revision", "NaN").toInt(&ok);
	if (!ok) rev = 0;

	emit(tradeUpdate(type, tradeId, actorId, rev));

	//Parse now all items
	QDomElement ti = t.firstChildElement();
	while (!ti.isNull()) {
		QString name = ti.nodeName();
		if (name == "tradeplayer") {
			int playerId = ti.attribute("playerid", "NaN").toInt(&ok);
			if (!ok) continue;

			bool accept = false;
			if (ti.attribute("accept", "0") == "1") accept = true;

			emit(tradePlayerUpdate(tradeId, playerId, accept));
		}
		if (name == "tradeestate") {
			int targetPlayerId = ti.attribute("targetplayer", "NaN").toInt(&ok);
			if (!ok) continue;

			int estateId = ti.attribute("estateid", "NaN").toInt(&ok);
			if (!ok) continue;

			emit(tradeEstateUpdate(tradeId, estateId, targetPlayerId));
		}
		if (name == "tradecard") {
			int targetPlayerId = ti.attribute("targetplayer", "NaN").toInt(&ok);
			if (!ok) continue;

			int cardId = ti.attribute("cardid", "NaN").toInt(&ok);
			if (!ok) continue;

			emit(tradeCardUpdate(tradeId, cardId, targetPlayerId));
		}
		if (name == "trademoney") {
			int fromPlayerId = ti.attribute("playerfrom", "NaN").toInt(&ok);
			if (!ok) continue;

			int targetPlayerId = ti.attribute("playerto", "NaN").toInt(&ok);
			if (!ok) continue;

			int money = ti.attribute("money", "NaN").toInt(&ok);
			if (!ok) continue;

			emit(tradeMoneyUpdate(tradeId, fromPlayerId, money, targetPlayerId));
		}
		ti = ti.nextSiblingElement();
	}

}

void MonopdClient::parseAuction(QDomElement a) {
	QString val;
	bool ok;

	//Check, if a auction has started
	int actor = -1;
	val = a.attribute("actor", "");
	if (val != "") {
		actor = val.toInt(&ok);
		if (ok && (actor > -1)) emit(auctionStarted());
	}

	int auctionid;
	val = a.attribute("auctionid", "");
	if (val != "") {
		auctionid = val.toInt(&ok);
		if (ok) emit(auctionIdUpdate(auctionid));
	}

	int fieldid;
	val = a.attribute("estateid", "");
	if (val != "") {
		fieldid = val.toInt(&ok);
		if (ok) emit(auctionFieldUpdate(fieldid));
	}

	int highbid;
	int highbidder;
	val = a.attribute("highbid", "");
	QString val2 = a.attribute("highbidder", "");
	if ((val != "") && (val2 != "")) {
		bool ok2;
		highbidder = val2.toInt(&ok2);
		highbid = val.toInt(&ok);
		if (ok && ok2) emit(auctionBidUpdate(highbidder, highbid));
	}

	int status;
	val = a.attribute("status", "");
	if (val != "") {
		status = val.toInt(&ok);
		if (ok) emit(auctionStatusUpdate(status));
	}
}

void MonopdClient::parseGame(QDomElement g) {
	int gameId;
	QString type;
	QString name;
	QString desc;
	QString status;
	int numPlayer;
	bool canBeJoined;
	int minPlayer;
	int maxPlayer;
	int master;

	bool ok;
	QString val = g.attribute("gameid", "NaN");
	if (val == "NaN") return;
	gameId = val.toInt(&ok);
	if (!ok) return;
	type = g.attribute("gametype", "");
	name = g.attribute("name", "");
	desc = g.attribute("description", "");
	if (gameId == -1) {
		MonopdTemplate* temp = getTemplate(type);
		if (name != "") temp->setName(name);
		if (desc != "") temp->setDesc(desc);
	} else {
		type = g.attribute("type", "");
		MonopdGame* game = getGame(gameId, type, true);
		if (game == 0) return;

		if (type != "") game->setType(type);

		val = g.attribute("name", "");
		if (val != "") game->setName(val);

		val = g.attribute("description", "");
		if (val != "") game->setName(val);

		val = g.attribute("status", "");
		if (val != "") {
			game->setStatus(val);
			if (gameId == curGameId) emit(statusChanged(game->getStatus()));
		}

		numPlayer = g.attribute("players", "").toInt(&ok);
		if (ok) game->setNumPlayer(numPlayer);

		val = g.attribute("canbejoined", "");
		if (val != "") {
			canBeJoined = false;
			if (val == "1") canBeJoined = true;
			game->setCanBeJoined(canBeJoined);
		}

		minPlayer = g.attribute("minplayers", "").toInt(&ok);
		if (ok) game->setMinPlayer(minPlayer);

		maxPlayer = g.attribute("maxplayers", "").toInt(&ok);
		if (ok) game->setMaxPlayer(maxPlayer);

		master = g.attribute("master", "").toInt(&ok);
		if (ok) game->setMinPlayer(master);
	}
}

void MonopdClient::parseConfig(QDomElement c) {
	QString val;
	bool ok;
	int gameId;
	val = c.attribute("gameid", "NaN");
	if (val != "NaN") {
		gameId = val.toInt(&ok);
		if (ok) {
			//We have old config updates. We have to parse them
			QDomElement e = c.firstChildElement("option");
			while (!e.isNull()) {
				parseConfigOption(e);
				e = e.nextSiblingElement("option");
			}
			return;
		}
	}

	//Here we have the new config updates
	QString cfgId;
	QString text;
	bool edit = false;
	bool value = false;

	cfgId = c.attribute("configid", "");
	if (cfgId == "") return;

	text = c.attribute("description", "");

	val = c.attribute("edit", "");
	if (val == "1") edit = true;

	val = c.attribute("value", "");
	if (val == "1") value = true;

	emit(configUpdate(".gc"+cfgId+":", text, value, edit));
}

//This method parses the old config updates
void MonopdClient::parseConfigOption(QDomElement o) {
	//title="Free Parking collects fines" command=".gef" value="0" edit="1"
	QString val;

	QString cmd;
	QString title;
	bool value = false;
	bool edit = false;

	cmd = o.attribute("command", "");
	if (cmd == "") return;

	title = o.attribute("title", "");

	val = o.attribute("value", "");
	if (val == "1") value = true;

	val = o.attribute("edit", "");
	if (val == "1") edit = true;

	emit(configUpdate(cmd, title, value, edit));
}

void MonopdClient::showData(QString data) {
	m_ui->textBrowser->append("From Server: "+data);
}

void MonopdClient::readData() {
	while (soket->canReadLine()) {
		QByteArray bytes = soket->readLine();
		int size = bytes.size();
		QString xml = QString::fromUtf8(bytes.data(), size);
		if (xml.startsWith("<monopd>")) cache = xml;
		else cache.append(xml);

		if (xml.endsWith("</monopd>\n")) {
			showData(cache);

			QDomDocument d;
			d.setContent(cache);
			parseDoc(d);
		}
	}
}

void MonopdClient::readServerList() {
	m_ui->tServer->clearContents();
	m_ui->tServer->setColumnCount(5);
	m_ui->tServer->setHorizontalHeaderLabels(QStringList("Name") << "Port" << "Players" << "Ip" << "Version");
	QByteArray bytes = httpSok->readAll();
	int size = bytes.size();
	QString xml = QString::fromUtf8(bytes.data(), size);
	QDomDocument d;
	showData(xml);
	d.setContent(xml);
	parseDoc(d);
	m_ui->tServer->resizeColumnsToContents();
}

void MonopdClient::sendData() {
	send(m_ui->lineEdit->text());
}

void MonopdClient::showError() {
	QMessageBox::critical(this, "Error", soket->errorString());
}

void MonopdClient::setServer(int r, int c) {
	m_ui->tfServer->setText(m_ui->tServer->item(r, 0)->text());
	m_ui->tfPort->setText(m_ui->tServer->item(r, 1)->text());
}

void MonopdClient::connectToServer() {
	QString host = m_ui->tfServer->text();
	int port;
	bool ok;
	port = m_ui->tfPort->text().toInt(&ok);
	if (!ok) return;

	connectToHost(host, port);

	if (soket->state() == QAbstractSocket::ConnectedState) {
		//hide();
		emit(connected());
	}
}

void MonopdClient::connectToHost(QString host, int port) {
	soket->connectToHost(host, port);
	soket->waitForConnected(2000);
	send(".nCapitalism");
	send(".pibell.png");
}

void MonopdClient::send(QString msg) {
	if (soket->state() != QAbstractSocket::ConnectedState) return;
	soket->write(msg.toUtf8()+"\n");
	soket->flush();
	m_ui->textBrowser->append("<span style=\"color:#000080\">To server: "+msg+"</span>\n");
}
